/* 
 * @file AIC3268Mode.c
 * 
 * @brief  Simple setup of AIC3268 for 8ch mixer
 *
 * @copyright Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/
 * All rights reserved. 
 * 
 * Software License Agreement
 * Texas Instruments (TI) is supplying this software for use solely and
 * exclusively on TI's Microcontroller and/or Data Converter products. 
 * The software is owned by TI and/or its suppliers, and is protected 
 * under applicable @copyright Copyright laws.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITH ALL FAULTS.
 * NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT
 * NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. TI SHALL NOT, UNDER ANY
 * CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 */ 
#include <driverlib.h>
#include <stdbool.h>
#include <string.h>
#include "reg_value_type.h"
#include "8channel_mixer.h"
#include "AIC3268Mode.h"
#include "i2c_driver.h"

// -----------------------------------------------------------------------------
// Audio_RegValWriteDSP
// -----------------------------------------------------------------------------
// Description:
//      Similar to Audio_AppWriteDSP but ignores miniDSP magic numbers
// Inputs:
//      const reg_value *data
//      unsigned int length_data
//      unsigned char address
// Outputs:
//      none
// -----------------------------------------------------------------------------
void Audio_RegValWriteDSP(const reg_value *data, unsigned int length_data, unsigned char address)
{
    int i;

    for(i=0; i<length_data; i++)
    {
        switch(data[i].reg_off)
        {
        case 254:                                   // Delay case
            Delay_ms(data[i].reg_val);
        break;

        case 255:                                   // Ignore this case
        break;

        default:
            writeI2C( address, data[i].reg_off, &(data[i].reg_val), 1 );
        }
    }
}
// -----------------------------------------------------------------------------
// Audio_AppWrite
// -----------------------------------------------------------------------------
// Description:
//      Parses a reg_value structure (typically  used for a non-miniDSP application).
// Inputs:
//      const reg_value *data
//      unsigned int length_data
//      unsigned char address
// Outputs:
//      none
// -----------------------------------------------------------------------------
void Audio_AppWrite(const reg_value *data, unsigned int length_data, unsigned char address)
{
    int i;

    for(i=0; i<length_data; i++)
    {
        writeI2C( address, data[i].reg_off, &(data[i].reg_val), 1 );
    }
}
// -----------------------------------------------------------------------------
// Audio_AppWriteDSP
// -----------------------------------------------------------------------------
// Description:
//      Parses a reg_value structure (typically used for a miniDSP application).
//      The reg_off magic number 254 calls a delay subroutine. A 255 calls
//      audio_regvalWrite to load:
//          - DSP_A_data (if reg_val = 0x00)
//          - DSP_D_data (if reg_val = 0x01)
//      This function also automatically determines if registers are contiguous
//      to perform auto-incremented writes.
// Inputs:
//      const reg_value *data
//      const reg_value *DSP_A_data
//      const reg_value *DSP_D_data
//      unsigned int length_data
//      unsigned int length_DSP_A
//      unsigned int length_DSP_D
//      unsigned char address
// Outputs:
//      none
// -----------------------------------------------------------------------------
void Audio_AppWriteDSP(const reg_value *data,
                    const reg_value *DSP_A_data,
                    const reg_value *DSP_D_data,
                    unsigned int length_data,
                    unsigned int length_DSP_A,
                    unsigned int length_DSP_D,
                    unsigned char address)
{
    int i;

    for(i=0; i<length_data; i++)
    {
        switch(data[i].reg_off)
        {
        case 254:                                   // Delay case
            Delay_ms(data[i].reg_val);
            break;

        case 255:                                   // miniDSP code case
            if(data[i].reg_val == 0x00)
            {
                Audio_RegValWriteDSP(DSP_A_data,
                             length_DSP_A, address);
            }
            else
            {
                if(data[i].reg_val == 0x01)
                    Audio_RegValWriteDSP(DSP_D_data,
                                 length_DSP_D, address);
            }
            break;

        default:
            writeI2C( address, data[i].reg_off, &(data[i].reg_val), 1 );
        }
    }
}

// -----------------------------------------------------------------------------
// AIC_Init
// -----------------------------------------------------------------------------
// Description:
//      Parses a reg_value structure (typically used for a miniDSP application).
// Inputs:
//      const reg_value *data
//      const reg_value *DSP_A_data
//      const reg_value *DSP_D_data
//      unsigned int length_data
//      unsigned int length_DSP_A
//      unsigned int length_DSP_D
//      unsigned char address
// Outputs:
//      none
void AIC_Init(unsigned char address)
{
    Audio_AppWriteDSP(                          // Load audio program
                    REG_Section_program,        // Configuration
                    miniDSP_A_reg_values,       // miniDSP_A C-RAM and I-RAM
                    miniDSP_D_reg_values,       // miniDSP_D C-RAM and I-RAM
                    sizeof(REG_Section_program)/2,
                    sizeof(miniDSP_A_reg_values)/2,
                    sizeof(miniDSP_D_reg_values)/2,
                    address);
}

// -----------------------------------------------------------------------------
// Delay_ms
// -----------------------------------------------------------------------------
// Description:
//      This function used for delay.
// Inputs:
//      unsigned int time
// Outputs:
//      none
// -----------------------------------------------------------------------------
void Delay_ms(unsigned int time)                     // Delay function in ms
{
    int i,h;

    for(i=0; i<time; i++)
    {
        for(h=0; h<420; h++){}
    }
}

